--[[
  Zorlen Library - Started by Marcus S. Zarra
  
  3.67.00
		All included spell functions that require caster form will now cast caster form if you are not in the required form
		castBearForm() will now use Dire Bear Form if that form is known
  
  3.66.00
		castHealingTouch() added by Jiral
		castUnderHealingTouch() added by Jiral
		castOverHealingTouch() added by Jiral
		castMaxHealingTouch() added by Jiral
		castRegrowth() added by Jiral
		castUnderRegrowth() added by Jiral
		castOverRegrowth() added by Jiral
		castMaxRegrowth() added by Jiral
		castRejuvenation() added by Jiral
		castUnderRejuvenation() added by Jiral
		castOverRejuvenation() added by Jiral
		castMaxRejuvenation() added by Jiral
  
  3.58.00
		Updated the druid functions to be able to use the action bar button slot information
  
  3.55.00
		castTravelForm() added by BigRedBrent
		castMoonkinForm() added by BigRedBrent
		castBearForm() added by BigRedBrent
		castDireBearForm() added by BigRedBrent
		castCatForm() added by BigRedBrent
		castAquaticForm() added by BigRedBrent
		isCasterForm() added by BigRedBrent
		isTravelForm() added by BigRedBrent
		isMoonkinForm() added by BigRedBrent
		isBearForm() added by BigRedBrent
		isDireBearForm() added by BigRedBrent
		isCatForm() added by BigRedBrent
		isAquaticForm() added by BigRedBrent
  
  3.53.00
		castThorns() added by BigRedBrent
		castSelfThorns() added by BigRedBrent
		isMarkOfTheWild() added by BigRedBrent
		isThornsActive() added by BigRedBrent
		isThorns() added by BigRedBrent
		Fixed: castMarkOfTheWild()
  
  3.17.00
		Zorlen_FormID() added by Jiral
		Zorlen_FormName() added by Jiral
  
  3.12.00
		castCasterForm() added by Jiral
  
  3.9.0
		isMarkOfTheWildActive() added by FelyzaChan
		castMarkOfTheWild() added by FelyzaChan
		castSelfMarkOfTheWild() added by FelyzaChan
  
  3.3.1
		Zorlen_RegisterDruidForm() added by BigRedBrent
		Zorlen_CheckDruidForm(form) added by BigRedBrent

  3.00  Rewrite by Wynn (Bleeding Hollow), break units into class functions.
		  
--]]


-- this is called when ever the main bar changes due to a shapeshift
-- Example: Zorlen_CurrentForm == "Bear Form"
-- The example above will return true if you are currently in the Bear Form.
function Zorlen_RegisterDruidForm()
	local i;
	local max = GetNumShapeshiftForms();
	for i = 1 , max do
		local _, name, isActive = GetShapeshiftFormInfo(i);
		if isActive then
			Zorlen_debug("You are now in "..name..".");
			Zorlen_CurrentForm = name;
			return;
		end
	end
	Zorlen_debug("You are now in Caster Form.");
	Zorlen_CurrentForm = "Caster Form";
end

-- Will return true if you are in the queried form.
-- Example: Zorlen_CheckDruidForm("Bear Form")
-- The example above will return true if you are currently in the Bear Form.
function Zorlen_CheckDruidForm(form)
	if form == Zorlen_CurrentForm then
		return true
	else
		return false
	end
end

function isCasterForm()
	if "Caster Form" == Zorlen_CurrentForm then
		return true
	else
		return false
	end
end

function isTravelForm()
	if LOCALIZATION_ZORLEN_TravelForm == Zorlen_CurrentForm then
		return true
	else
		return false
	end
end

function isMoonkinForm()
	if LOCALIZATION_ZORLEN_MoonkinForm == Zorlen_CurrentForm then
		return true
	else
		return false
	end
end

function isBearForm()
	if LOCALIZATION_ZORLEN_BearForm == Zorlen_CurrentForm then
		return true
	else
		return false
	end
end

function isDireBearForm()
	if LOCALIZATION_ZORLEN_DireBearForm == Zorlen_CurrentForm then
		return true
	else
		return false
	end
end

function isCatForm()
	if LOCALIZATION_ZORLEN_CatForm == Zorlen_CurrentForm then
		return true
	else
		return false
	end
end

function isAquaticForm()
	if LOCALIZATION_ZORLEN_AquaticForm == Zorlen_CurrentForm then
		return true
	else
		return false
	end
end



function castCasterForm()
	local SpellName = Zorlen_CurrentForm
	if not isCasterForm() then
		local SpellID = Zorlen_GetSpellID(SpellName)
		CastSpell(SpellID, 0)
		return true
	end
	return false
end

function castTravelForm()
	local SpellName = LOCALIZATION_ZORLEN_TravelForm
	local SpellButton = Zorlen_Button_TravelForm
	if SpellButton then
		local isUsable, notEnoughMana = IsUsableAction(SpellButton)
		local _, duration, _ = GetActionCooldown(SpellButton)
		local isCurrent = IsCurrentAction(SpellButton)
		if isCasterForm() and ( isUsable == 1 ) and ( not notEnoughMana ) and ( duration == 0 ) and not ( isCurrent == 1 ) and (not Zorlen_CheckDruidForm(SpellName)) then
			UseAction(SpellButton)
			return true
		else if not isCasterForm() and not isTravelForm() then
			return castCasterForm()
		end
		end
	else if Zorlen_IsSpellKnown(SpellName) then
		Zorlen_debug(""..SpellName.." was not found on any of the action bars!")
		if isCasterForm() and not Zorlen_CheckDruidForm(SpellName) then
			local SpellID = Zorlen_GetSpellID(SpellName)
			CastSpell(SpellID, 0)
			return true
		else if not isCasterForm() and not isTravelForm() then
			return castCasterForm()
		end
		end
	end
	end
	return false
end

function castMoonkinForm()
	local SpellName = LOCALIZATION_ZORLEN_MoonkinForm
	local SpellButton = Zorlen_Button_MoonkinForm
	if SpellButton then
		local isUsable, notEnoughMana = IsUsableAction(SpellButton)
		local _, duration, _ = GetActionCooldown(SpellButton)
		local isCurrent = IsCurrentAction(SpellButton)
		if isCasterForm() and ( isUsable == 1 ) and ( not notEnoughMana ) and ( duration == 0 ) and not ( isCurrent == 1 ) and (not Zorlen_CheckDruidForm(SpellName)) then
			UseAction(SpellButton)
			return true
		else if not isCasterForm() and not isMoonkinForm() then
			return castCasterForm()
		end
		end
	else if Zorlen_IsSpellKnown(SpellName) then
		Zorlen_debug(""..SpellName.." was not found on any of the action bars!")
		if isCasterForm() and not Zorlen_CheckDruidForm(SpellName) then
			local SpellID = Zorlen_GetSpellID(SpellName)
			CastSpell(SpellID, 0)
			return true
		else if not isCasterForm() and not isMoonkinForm() then
			return castCasterForm()
		end
		end
	end
	end
	return false
end

function castBearForm()
	if Zorlen_IsSpellKnown(LOCALIZATION_ZORLEN_DireBearForm) then
		return castDireBearForm()
	end
	local SpellName = LOCALIZATION_ZORLEN_BearForm
	local SpellButton = Zorlen_Button_BearForm
	if SpellButton then
		local isUsable, notEnoughMana = IsUsableAction(SpellButton)
		local _, duration, _ = GetActionCooldown(SpellButton)
		local isCurrent = IsCurrentAction(SpellButton)
		if isCasterForm() and ( isUsable == 1 ) and ( not notEnoughMana ) and ( duration == 0 ) and not ( isCurrent == 1 ) and (not Zorlen_CheckDruidForm(SpellName)) then
			UseAction(SpellButton)
			return true
		else if not isCasterForm() and not isBearForm() then
			return castCasterForm()
		end
		end
	else if Zorlen_IsSpellKnown(SpellName) then
		Zorlen_debug(""..SpellName.." was not found on any of the action bars!")
		if isCasterForm() and not Zorlen_CheckDruidForm(SpellName) then
			local SpellID = Zorlen_GetSpellID(SpellName)
			CastSpell(SpellID, 0)
			return true
		else if not isCasterForm() and not isBearForm() then
			return castCasterForm()
		end
		end
	end
	end
	return false
end

function castDireBearForm()
	local SpellName = LOCALIZATION_ZORLEN_DireBearForm
	local SpellButton = Zorlen_Button_DireBearForm
	if SpellButton then
		local isUsable, notEnoughMana = IsUsableAction(SpellButton)
		local _, duration, _ = GetActionCooldown(SpellButton)
		local isCurrent = IsCurrentAction(SpellButton)
		if isCasterForm() and ( isUsable == 1 ) and ( not notEnoughMana ) and ( duration == 0 ) and not ( isCurrent == 1 ) and (not Zorlen_CheckDruidForm(SpellName)) then
			UseAction(SpellButton)
			return true
		else if not isCasterForm() and not isDireBearForm() then
			return castCasterForm()
		end
		end
	else if Zorlen_IsSpellKnown(SpellName) then
		Zorlen_debug(""..SpellName.." was not found on any of the action bars!")
		if isCasterForm() and not Zorlen_CheckDruidForm(SpellName) then
			local SpellID = Zorlen_GetSpellID(SpellName)
			CastSpell(SpellID, 0)
			return true
		else if not isCasterForm() and not isDireBearForm() then
			return castCasterForm()
		end
		end
	end
	end
	return false
end

function castCatForm()
	local SpellName = LOCALIZATION_ZORLEN_CatForm
	local SpellButton = Zorlen_Button_CatForm
	if SpellButton then
		local isUsable, notEnoughMana = IsUsableAction(SpellButton)
		local _, duration, _ = GetActionCooldown(SpellButton)
		local isCurrent = IsCurrentAction(SpellButton)
		if isCasterForm() and ( isUsable == 1 ) and ( not notEnoughMana ) and ( duration == 0 ) and not ( isCurrent == 1 ) and (not Zorlen_CheckDruidForm(SpellName)) then
			UseAction(SpellButton)
			return true
		else if not isCasterForm() and not isCatForm() then
			return castCasterForm()
		end
		end
	else if Zorlen_IsSpellKnown(SpellName) then
		Zorlen_debug(""..SpellName.." was not found on any of the action bars!")
		if isCasterForm() and not Zorlen_CheckDruidForm(SpellName) then
			local SpellID = Zorlen_GetSpellID(SpellName)
			CastSpell(SpellID, 0)
			return true
		else if not isCasterForm() and not isCatForm() then
			return castCasterForm()
		end
		end
	end
	end
	return false
end

function castAquaticForm()
	local SpellName = LOCALIZATION_ZORLEN_AquaticForm
	local SpellButton = Zorlen_Button_AquaticForm
	if SpellButton then
		local isUsable, notEnoughMana = IsUsableAction(SpellButton)
		local _, duration, _ = GetActionCooldown(SpellButton)
		local isCurrent = IsCurrentAction(SpellButton)
		if isCasterForm() and ( isUsable == 1 ) and ( not notEnoughMana ) and ( duration == 0 ) and not ( isCurrent == 1 ) and (not Zorlen_CheckDruidForm(SpellName)) then
			UseAction(SpellButton)
			return true
		else if not isCasterForm() and not isAquaticForm() then
			return castCasterForm()
		end
		end
	else if Zorlen_IsSpellKnown(SpellName) then
		Zorlen_debug(""..SpellName.." was not found on any of the action bars!")
		if isCasterForm() and not Zorlen_CheckDruidForm(SpellName) then
			local SpellID = Zorlen_GetSpellID(SpellName)
			CastSpell(SpellID, 0)
			return true
		else if not isCasterForm() and not isAquaticForm() then
			return castCasterForm()
		end
		end
	end
	end
	return false
end



function Zorlen_FormID()
	if Zorlen_CurrentForm and not (Zorlen_CurrentForm == "Caster Form") then
		local SpellName = Zorlen_CurrentForm
		local SpellID = Zorlen_GetSpellID(SpellName)
		return SpellID
	end
	return nil
end



function Zorlen_FormName()
	if Zorlen_CurrentForm and not (Zorlen_CurrentForm == "Caster Form") then
		local SpellName = Zorlen_CurrentForm
		return SpellName
	end
	return nil
end




--Returns true if Mark of the Wild is active
function isMarkOfTheWildActive()
	return Zorlen_checkBuff("Spell_Nature_Regeneration")
end

--Returns true if Thorns is active
function isThornsActive()
	return Zorlen_checkBuff("Spell_Nature_Thorns")
end

--Returns true if Mark of the Wild is active on the target
function isMarkOfTheWild(unit, castable)
	local u = "target"
	if unit then
		u = unit
	end
	return Zorlen_checkBuff("Spell_Nature_Regeneration", u, castable)
end

--Returns true if Thorns is active on the target
function isThorns(unit, castable)
	local u = "target"
	if unit then
		u = unit
	end
	return Zorlen_checkBuff("Spell_Nature_Thorns", u, castable)
end




-- Will cast the spell if it is not on your target, if it is on your target or it can not be cast on your target, then it will cast on yourself if it is not on you.
function castMarkOfTheWild()
	if not isCasterForm() then
		return castCasterForm()
	else
		local SpellName = LOCALIZATION_ZORLEN_MarkOfTheWild
		local SpellButton = Zorlen_Button_MarkOfTheWild
		local friend = nil;
		if SpellButton then
			if isCasterForm() then
				local isUsable, notEnoughMana = IsUsableAction(SpellButton)
				local _, duration, _ = GetActionCooldown(SpellButton)
				local inRange = IsActionInRange(SpellButton)
				local isCurrent = IsCurrentAction(SpellButton)
				if (UnitIsFriend("player", "target") and not UnitIsUnit("player","target") and ( isUsable == 1 ) and ( not notEnoughMana ) and ( duration == 0 ) and ( inRange == 1 ) and not ( isCurrent == 1 ) and (not isMarkOfTheWild())) then
					UseAction(SpellButton)
					return true
				else if ( isUsable == 1 ) and ( not notEnoughMana ) and ( duration == 0 ) and not ( isCurrent == 1 ) and (not isMarkOfTheWildActive()) then
					if (UnitIsFriend("player", "target") and not UnitIsUnit("player","target")) then
						friend = 1;
						TargetUnit("player");
					end
					UseAction(SpellButton)
					if (SpellIsTargeting() and SpellCanTargetUnit("player")) then
						SpellTargetUnit("player");
					end
					if (friend) then
						TargetLastTarget();
					end
					return true
				end
				end
			end
		else if Zorlen_IsSpellKnown(SpellName) then
			Zorlen_debug(""..SpellName.." was not found on any of the action bars!")
			if isCasterForm() then
				local SpellID = Zorlen_GetSpellID(SpellName)
				if (UnitIsFriend("player", "target") and not UnitIsUnit("player","target") and not isMarkOfTheWild()) then
					CastSpell(SpellID, 0)
					return true
				else if (not isMarkOfTheWildActive()) then
					if (UnitIsFriend("player", "target") and not UnitIsUnit("player","target")) then
						friend = 1;
						TargetUnit("player");
					end
					CastSpell(SpellID, 0)
					if (SpellIsTargeting() and SpellCanTargetUnit("player")) then
						SpellTargetUnit("player");
					end
					if (friend) then
						TargetLastTarget();
					end
					return true
				end
				end
			end
		end
		end
	end
	return false
end



-- Will only cast the spell on your self if you do not have it on you and will not be able to cast on anything else.
function castSelfMarkOfTheWild()
	if not isCasterForm() then
		return castCasterForm()
	else
		local SpellName = LOCALIZATION_ZORLEN_MarkOfTheWild
		local SpellButton = Zorlen_Button_MarkOfTheWild
		local friend = nil;
		if SpellButton then
			if isCasterForm() then
				local isUsable, notEnoughMana = IsUsableAction(SpellButton)
				local _, duration, _ = GetActionCooldown(SpellButton)
				local isCurrent = IsCurrentAction(SpellButton)
				if ( isUsable == 1 ) and ( not notEnoughMana ) and ( duration == 0 ) and not ( isCurrent == 1 ) and (not isMarkOfTheWildActive()) then
					if (UnitIsFriend("player", "target") and not UnitIsUnit("player","target")) then
						friend = 1;
						TargetUnit("player");
					end
					UseAction(SpellButton)
					if (SpellIsTargeting() and SpellCanTargetUnit("player")) then
						SpellTargetUnit("player");
					end
					if (friend) then
						TargetLastTarget();
					end
					return true
				end
			end
		else if Zorlen_IsSpellKnown(SpellName) then
			Zorlen_debug(""..SpellName.." was not found on any of the action bars!")
			if isCasterForm() then
				local SpellID = Zorlen_GetSpellID(SpellName);
				if (not isMarkOfTheWildActive()) then
					if (UnitIsFriend("player", "target") and not UnitIsUnit("player","target")) then
						friend = 1;
						TargetUnit("player");
					end
					CastSpell(SpellID, 0)
					if (SpellIsTargeting() and SpellCanTargetUnit("player")) then
						SpellTargetUnit("player");
					end
					if (friend) then
						TargetLastTarget();
					end
					return true
				end
			end
		end
		end
	end
	return false
end


-- Will cast the spell if it is not on your target, if it is on your target or it can not be cast on your target, then it will cast on yourself if it is not on you.
function castThorns()
	if not isCasterForm() then
		return castCasterForm()
	else
		local SpellName = LOCALIZATION_ZORLEN_Thorns
		local SpellButton = Zorlen_Button_Thorns
		local friend = nil;
		if SpellButton then
			if isCasterForm() then
				local isUsable, notEnoughMana = IsUsableAction(SpellButton)
				local _, duration, _ = GetActionCooldown(SpellButton)
				local inRange = IsActionInRange(SpellButton)
				local isCurrent = IsCurrentAction(SpellButton)
				if (UnitIsFriend("player", "target") and not UnitIsUnit("player","target") and ( isUsable == 1 ) and ( not notEnoughMana ) and ( duration == 0 ) and ( inRange == 1 ) and not ( isCurrent == 1 ) and (not isThorns())) then
					UseAction(SpellButton)
					return true
				else if ( isUsable == 1 ) and ( not notEnoughMana ) and ( duration == 0 ) and not ( isCurrent == 1 ) and (not isThornsActive()) then
					if (UnitIsFriend("player", "target") and not UnitIsUnit("player","target")) then
						friend = 1;
						TargetUnit("player");
					end
					UseAction(SpellButton)
					if (SpellIsTargeting() and SpellCanTargetUnit("player")) then
						SpellTargetUnit("player");
					end
					if (friend) then
						TargetLastTarget();
					end
					return true
				end
				end
			end
		else if Zorlen_IsSpellKnown(SpellName) then
			Zorlen_debug(""..SpellName.." was not found on any of the action bars!")
			if isCasterForm() then
				local SpellID = Zorlen_GetSpellID(SpellName)
				if (UnitIsFriend("player", "target") and not UnitIsUnit("player","target") and not isThorns()) then
					CastSpell(SpellID, 0)
					return true
				else if (not isThornsActive()) then
					if (UnitIsFriend("player", "target") and not UnitIsUnit("player","target")) then
						friend = 1;
						TargetUnit("player");
					end
					CastSpell(SpellID, 0)
					if (SpellIsTargeting() and SpellCanTargetUnit("player")) then
						SpellTargetUnit("player");
					end
					if (friend) then
						TargetLastTarget();
					end
					return true
				end
				end
			end
		end
		end
	end
	return false
end



-- Will only cast the spell on your self if you do not have it on you and will not be able to cast on anything else.
function castSelfThorns()
	if not isCasterForm() then
		return castCasterForm()
	else
		local SpellName = LOCALIZATION_ZORLEN_Thorns
		local SpellButton = Zorlen_Button_Thorns
		local friend = nil;
		if SpellButton then
			if isCasterForm() then
				local isUsable, notEnoughMana = IsUsableAction(SpellButton)
				local _, duration, _ = GetActionCooldown(SpellButton)
				local isCurrent = IsCurrentAction(SpellButton)
				if ( isUsable == 1 ) and ( not notEnoughMana ) and ( duration == 0 ) and not ( isCurrent == 1 ) and (not isThornsActive()) then
					if (UnitIsFriend("player", "target") and not UnitIsUnit("player","target")) then
						friend = 1;
						TargetUnit("player");
					end
					UseAction(SpellButton)
					if (SpellIsTargeting() and SpellCanTargetUnit("player")) then
						SpellTargetUnit("player");
					end
					if (friend) then
						TargetLastTarget();
					end
					return true
				end
			end
		else if Zorlen_IsSpellKnown(SpellName) then
			Zorlen_debug(""..SpellName.." was not found on any of the action bars!")
			if isCasterForm() then
				local SpellID = Zorlen_GetSpellID(SpellName);
				if (not isThornsActive()) then
					if (UnitIsFriend("player", "target") and not UnitIsUnit("player","target")) then
						friend = 1;
						TargetUnit("player");
					end
					CastSpell(SpellID, 0)
					if (SpellIsTargeting() and SpellCanTargetUnit("player")) then
						SpellTargetUnit("player");
					end
					if (friend) then
						TargetLastTarget();
					end
					return true
				end
			end
		end
		end
	end
	return false
end










-- From: Jiral
-- Casts Healing Touch of a rank appropriate to the damage taken.
--   Note: WoW doesn't return actual health numbers for non-self/party/raid members (only percents), so maximum rank will be used by default
function castHealingTouch(Mode,RankAdj)
	if not isCasterForm() then
		return castCasterForm()
	else
		local SpellName = LOCALIZATION_ZORLEN_HealingTouch
		local LevelLearnedArray={1,8,14,20,26,32,38,44,50,56,60}
		--local ManaArray={22,50,100,168,245,304,368,450,546,655,880} -- All but last rank is Mana cost from 1.10 test server
		local ManaArray={30,60,120,205,300,370,445,545,660,790,880}
		local MinHealArray={40,094,195,363,572,742,0936,1199,1516,1890,2267}
		local MaxHealArray={55,119,243,445,694,894,1120,1427,1796,2230,2677}
		local TimeArray={1.5,2.0,2.5,3.0,3.5,3.5,3.5,3.5,3.5,3.5,3.5}
		if Zorlen_HasTalent(LOCALIZATION_ZORLEN_TranquilSpirit) then
			local ManaReduction = 1 - (Zorlen_GetTalentRank(LOCALIZATION_ZORLEN_TranquilSpirit) * .02)
			for i=1,11 do
				ManaArray[i] = (ManaArray[i] * ManaReduction) + 1
			end		
		end
		if Zorlen_HasTalent(LOCALIZATION_ZORLEN_GiftofNature) then
			local HealImproved = 1 + (Zorlen_GetTalentRank(LOCALIZATION_ZORLEN_GiftofNature) * .02)
			for i=1,SpellRanks do
				MinHealArray[i] = MinHealArray[i] * HealImproved
				MaxHealArray[i] = MaxHealArray[i] * HealImproved
			end
		end
		return Zorlen_CastHealingSpell(SpellName,ManaArray,MinHealArray,MaxHealArray,TimeArray,LevelLearnedArray,Mode,RankAdj)
	end
end

-- From: Jiral
function castUnderHealingTouch(RankAdj)
	local DefaultAdj = -1
	if RankAdj then
		DefaultAdj = RankAdj
	end
	return castHealingTouch("under",DefaultAdj)
end

-- From: Jiral
function castOverHealingTouch(RankAdj)
	local DefaultAdj = 1
	if RankAdj then
		DefaultAdj = RankAdj
	end
	return castHealingTouch("over",DefaultAdj)
end

-- From: Jiral
function castMaxHealingTouch(RankAdj)
	return castHealingTouch("maximum",RankAdj)
end

-- From: Jiral
function castRegrowth(Mode,RankAdj)
	if not isCasterForm() then
		return castCasterForm()
	else
		local SpellName = LOCALIZATION_ZORLEN_Regrowth
		local TargetID = "player"
		if UnitExists("target") and not UnitCanAttack("player","target") then
			TargetID = "target"
		end
		if not Zorlen_checkBuffBySpellName(SpellName, TargetID) then
			local LevelLearnedArray={12,18,24,30,36,42,48,54,60}
			local ManaArray={120,205,280,350,420,510,615,740,880}
			local MinHealArray={182,339,499,661,832,1057,1332,1670,2067}
			local MaxHealArray={196,363,533,661,884,1121,1410,1766,2183}
			local TimeArray={2,2,2,2,2,2,2,2,2}
			if Zorlen_HasTalent(LOCALIZATION_ZORLEN_GiftofNature) then
				local HealImproved = 1 + (Zorlen_GetTalentRank(LOCALIZATION_ZORLEN_GiftofNature) * .02)
				for i=1,SpellRanks do
					MinHealArray[i] = MinHealArray[i] * HealImproved
					MaxHealArray[i] = MaxHealArray[i] * HealImproved
				end
			end
			return Zorlen_CastHealingSpell(SpellName,ManaArray,MinHealArray,MaxHealArray,TimeArray,LevelLearnedArray,Mode,RankAdj)
		else
			return false
		end
	end
end

-- From: Jiral
function castUnderRegrowth(RankAdj)
	local DefaultAdj = -1
	if RankAdj then
		DefaultAdj = RankAdj
	end
	return castRegrowth("under",DefaultAdj)
end

-- From: Jiral
function castOverRegrowth(RankAdj)
	local DefaultAdj = 1
	if RankAdj then
		DefaultAdj = RankAdj
	end
	return castRegrowth("over",DefaultAdj)
end

-- From: Jiral
function castMaxRegrowth(RankAdj)
	return castRegrowth("maximum",RankAdj)
end

-- From: Jiral
function castRejuvenation(Mode,RankAdj)
	if not isCasterForm() then
		return castCasterForm()
	else
		local SpellName = LOCALIZATION_ZORLEN_Rejuvenation
		local TargetID = "player"
		if UnitExists("target") and not UnitCanAttack("player","target") then
			TargetID = "target"
		end
		if not Zorlen_checkBuffBySpellName(SpellName, TargetID) then
			local LevelLearnedArray={4,10,16,22,28,34,40,46,52,58,60}
			local ManaArray={25,40,75,105,135,160,195,235,280,335,360}
			local MinHealArray={32,56,116,180,244,304,388,488,608,756,888}
			local MaxHealArray={32,56,116,180,244,304,388,488,608,756,888}
			local TimeArray={0,0,0,0,0,0,0,0,0,0,0}
			local HealImproved = 1
			if Zorlen_HasTalent(LOCALIZATION_ZORLEN_GiftofNature) then
				HealImproved = HealImproved + (Zorlen_GetTalentRank(LOCALIZATION_ZORLEN_GiftofNature) * .02)
			end
			if Zorlen_HasTalent(LOCALIZATION_ZORLEN_ImprovedRejuvenation) then
				HealImproved = HealImproved + (Zorlen_GetTalentRank(LOCALIZATION_ZORLEN_ImprovedRejuvenation) * .05)
			end
			if HealImproved > 1 then
				for i=1,11 do
					MinHealArray[i] = MinHealArray[i] * HealImproved
					MaxHealArray[i] = MaxHealArray[i] * HealImproved
				end
			end
			return Zorlen_CastHealingSpell(SpellName,ManaArray,MinHealArray,MaxHealArray,TimeArray,LevelLearnedArray,Mode,RankAdj)
		else
			return false
		end
	end
end

-- From: Jiral
function castUnderRejuvenation(RankAdj)
	local DefaultAdj = -1
	if RankAdj then
		DefaultAdj = RankAdj
	end
	return castRejuvenation("under",DefaultAdj)
end

-- From: Jiral
function castOverRejuvenation(RankAdj)
	local DefaultAdj = 1
	if RankAdj then
		DefaultAdj = RankAdj
	end
	return castRejuvenation("over",DefaultAdj)
end

-- From: Jiral
function castMaxRejuvenation(RankAdj)
	return castRejuvenation("maximum",RankAdj)
end



